@@ -16,9 +16,6 @@
 #################################################################################*/
 #ifndef _ica_H
 #define _ica_H
-#include <R.h>
-#include <Rmath.h>
-#include <math.h>
 #include <RcppArmadillo.h>
 RcppExport SEXP radicalrot(SEXP , SEXP);
 arma::mat fs10(arma::mat X, arma::mat B, const double nsamples);
